@@ -1,20 +1,20 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * GData Client
- * Copyright (C) Philip Withnall 2008 <philip@tecnocode.co.uk>
- * 
- * GData Client is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * Copyright (C) Philip Withnall 2008-2009 <philip@tecnocode.co.uk>
+ *
+ * GData Client is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * GData Client is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GData Client.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with GData Client.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <glib.h>
@@ -53,7 +53,7 @@ test_authentication (void)
 
 	/* Check all is as it should be */
 	g_assert (gdata_service_is_authenticated (service) == TRUE);
-	g_assert_cmpstr (gdata_service_get_username (service), ==, YT_USERNAME);
+	g_assert_cmpstr (gdata_service_get_username (service), ==, YT_USERNAME "@gmail.com");
 	g_assert_cmpstr (gdata_service_get_password (service), ==, YT_PASSWORD);
 	g_assert_cmpstr (gdata_youtube_service_get_youtube_user (GDATA_YOUTUBE_SERVICE (service)), ==, YT_USERNAME);
 }
@@ -73,7 +73,7 @@ test_authentication_async_cb (GDataService *service, GAsyncResult *async_result,
 
 	/* Check all is as it should be */
 	g_assert (gdata_service_is_authenticated (service) == TRUE);
-	g_assert_cmpstr (gdata_service_get_username (service), ==, YT_USERNAME);
+	g_assert_cmpstr (gdata_service_get_username (service), ==, YT_USERNAME "@gmail.com");
 	g_assert_cmpstr (gdata_service_get_password (service), ==, YT_PASSWORD);
 	g_assert_cmpstr (gdata_youtube_service_get_youtube_user (GDATA_YOUTUBE_SERVICE (service)), ==, YT_USERNAME);
 }
@@ -277,7 +277,7 @@ test_upload_simple (void)
 
 	g_assert (service != NULL);
 
-	video = gdata_youtube_video_new ();
+	video = gdata_youtube_video_new (NULL);
 
 	gdata_entry_set_title (GDATA_ENTRY (video), "Bad Wedding Toast");
 	gdata_youtube_video_set_title (video, "Bad Wedding Toast");
@@ -291,6 +291,7 @@ test_upload_simple (void)
 	g_assert_cmpstr (xml, ==,
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 				"xmlns:media='http://search.yahoo.com/mrss/' "
+				"xmlns:gd='http://schemas.google.com/g/2005' "
 				"xmlns:yt='http://gdata.youtube.com/schemas/2007'>"
 			 	"<title type='text'>Bad Wedding Toast</title>"
 			 	"<media:group>"
@@ -298,7 +299,6 @@ test_upload_simple (void)
 			 		"<media:title type='plain'>Bad Wedding Toast</media:title>"
 			 		"<media:description type='plain'>I gave a bad toast at my friend&apos;s wedding.</media:description>"
 			 		"<media:keywords>toast, wedding</media:keywords>"
-			 		"<yt:duration seconds='0'/>"
 			 	"</media:group>"
 			 "</entry>");
 	g_free (xml);
@@ -371,6 +371,212 @@ test_parsing_app_control (void)
 	g_object_unref (video);
 }
 
+static void
+test_parsing_yt_recorded (void)
+{
+	GDataYouTubeVideo *video;
+	GTimeVal recorded;
+	gchar *xml;
+	GError *error = NULL;
+
+	video = gdata_youtube_video_new_from_xml (
+		"<entry xmlns='http://www.w3.org/2005/Atom' "
+			"xmlns:media='http://search.yahoo.com/mrss/' "
+			"xmlns:yt='http://gdata.youtube.com/schemas/2007' "
+			"xmlns:gd='http://schemas.google.com/g/2005' "
+			"gd:etag='W/\"CEMFSX47eCp7ImA9WxVUGEw.\"'>"
+			"<id>tag:youtube.com,2008:video:JAagedeKdcQ</id>"
+			"<published>2006-05-16T14:06:37.000Z</published>"
+			"<updated>2009-03-23T12:46:58.000Z</updated>"
+			"<category scheme='http://schemas.google.com/g/2005#kind' term='http://gdata.youtube.com/schemas/2007#video'/>"
+			"<title>Judas Priest - Painkiller</title>"
+			"<link rel='alternate' type='text/html' href='http://www.youtube.com/watch?v=JAagedeKdcQ'/>"
+			"<link rel='self' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ?client=ytapi-google-jsdemo'/>"
+			"<author>"
+				"<name>eluves</name>"
+				"<uri>http://gdata.youtube.com/feeds/api/users/eluves</uri>"
+			"</author>"
+			"<media:group>"
+				"<media:title type='plain'>Judas Priest - Painkiller</media:title>"
+				"<media:credit role='uploader' scheme='urn:youtube'>eluves</media:credit>"
+				"<media:category label='Music' scheme='http://gdata.youtube.com/schemas/2007/categories.cat'>Music</media:category>"
+			"</media:group>"
+			"<yt:recorded>2003-08-03</yt:recorded>"
+		"</entry>", -1, &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_YOUTUBE_VIDEO (video));
+	g_clear_error (&error);
+
+	/* Test the recorded date */
+	gdata_youtube_video_get_recorded (video, &recorded);
+	g_assert_cmpint (recorded.tv_sec, ==, 1059868800);
+	g_assert_cmpint (recorded.tv_usec, ==, 0);
+
+	/* Update the state and see if the XML's written out OK */
+	recorded.tv_sec = 1128229200;
+	gdata_youtube_video_set_recorded (video, &recorded);
+
+	/* Check the XML */
+	xml = gdata_entry_get_xml (GDATA_ENTRY (video));
+	g_assert_cmpstr (xml, ==,
+			 "<entry xmlns='http://www.w3.org/2005/Atom' "
+				"xmlns:media='http://search.yahoo.com/mrss/' "
+				"xmlns:gd='http://schemas.google.com/g/2005' "
+				"xmlns:yt='http://gdata.youtube.com/schemas/2007' "
+				"gd:etag='W/\"CEMFSX47eCp7ImA9WxVUGEw.\"'>"
+				"<title type='text'>Judas Priest - Painkiller</title>"
+				"<id>tag:youtube.com,2008:video:JAagedeKdcQ</id>"
+				"<updated>2009-03-23T12:46:58Z</updated>"
+				"<published>2006-05-16T14:06:37Z</published>"
+				"<category term='http://gdata.youtube.com/schemas/2007#video' scheme='http://schemas.google.com/g/2005#kind'/>"
+				"<link href='http://www.youtube.com/watch?v=JAagedeKdcQ' rel='alternate' type='text/html'/>"
+				"<link href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ?client=ytapi-google-jsdemo' rel='self' type='application/atom+xml'/>"
+				"<author>"
+					"<name>eluves</name>"
+					"<uri>http://gdata.youtube.com/feeds/api/users/eluves</uri>"
+				"</author>"
+				"<media:group>"
+					"<media:category label='Music' scheme='http://gdata.youtube.com/schemas/2007/categories.cat'>Music</media:category>"
+					"<media:title type='plain'>Judas Priest - Painkiller</media:title>"
+				"</media:group>"
+				"<yt:recorded>2005-10-02</yt:recorded>"
+			 "</entry>");
+	g_free (xml);
+
+	/* TODO: more tests on entry properties */
+
+	g_object_unref (video);
+}
+
+static void
+test_parsing_comments_feed_link (void)
+{
+	GDataYouTubeVideo *video;
+	GDataGDFeedLink *feed_link;
+	GError *error = NULL;
+
+	video = gdata_youtube_video_new_from_xml (
+		"<entry xmlns='http://www.w3.org/2005/Atom' "
+			"xmlns:media='http://search.yahoo.com/mrss/' "
+			"xmlns:yt='http://gdata.youtube.com/schemas/2007' "
+			"xmlns:gd='http://schemas.google.com/g/2005' "
+			"gd:etag='W/\"CEMFSX47eCp7ImA9WxVUGEw.\"'>"
+			"<id>tag:youtube.com,2008:video:JAagedeKdcQ</id>"
+			"<published>2006-05-16T14:06:37.000Z</published>"
+			"<updated>2009-03-23T12:46:58.000Z</updated>"
+			"<category scheme='http://schemas.google.com/g/2005#kind' term='http://gdata.youtube.com/schemas/2007#video'/>"
+			"<title>Judas Priest - Painkiller</title>"
+			"<link rel='alternate' type='text/html' href='http://www.youtube.com/watch?v=JAagedeKdcQ'/>"
+			"<link rel='self' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ?client=ytapi-google-jsdemo'/>"
+			"<author>"
+				"<name>eluves</name>"
+				"<uri>http://gdata.youtube.com/feeds/api/users/eluves</uri>"
+			"</author>"
+			"<media:group>"
+				"<media:title type='plain'>Judas Priest - Painkiller</media:title>"
+				"<media:credit role='uploader' scheme='urn:youtube'>eluves</media:credit>"
+				"<media:category label='Music' scheme='http://gdata.youtube.com/schemas/2007/categories.cat'>Music</media:category>"
+			"</media:group>"
+			"<gd:comments>"
+				"<gd:feedLink href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ/comments' countHint='13021'/>"
+			"</gd:comments>"
+		"</entry>", -1, &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_YOUTUBE_VIDEO (video));
+	g_clear_error (&error);
+
+	/* Test the feed link */
+	feed_link = gdata_youtube_video_get_comments_feed_link (video);
+	g_assert (feed_link != NULL);
+	g_assert (feed_link->rel == NULL);
+	g_assert_cmpstr (feed_link->href, ==, "http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ/comments");
+	g_assert_cmpuint (feed_link->count_hint, ==, 13021);
+	g_assert (feed_link->read_only == FALSE);
+
+	/* TODO: more tests on entry properties */
+
+	g_object_unref (video);
+}
+
+static void
+test_query_uri (void)
+{
+	gdouble latitude, longitude, radius;
+	gboolean has_location;
+	gchar *query_uri;
+	GDataYouTubeQuery *query = gdata_youtube_query_new ("q");
+
+	gdata_youtube_query_set_format (query, GDATA_YOUTUBE_FORMAT_RTSP_H263_AMR);
+	g_assert_cmpuint (gdata_youtube_query_get_format (query), ==, 1);
+
+	/* Location */
+	gdata_youtube_query_set_location (query, 45.01364, -97.12356, 112.5, TRUE);
+	gdata_youtube_query_get_location (query, &latitude, &longitude, &radius, &has_location);
+
+	g_assert_cmpfloat (latitude, ==, 45.01364);
+	g_assert_cmpfloat (longitude, ==, -97.12356);
+	g_assert_cmpfloat (radius, ==, 112.5);
+	g_assert (has_location == TRUE);
+
+	query_uri = gdata_query_get_query_uri (GDATA_QUERY (query), "http://example.com");
+	g_assert_cmpstr (query_uri, ==, "http://example.com?q=q&time=all_time&safeSearch=none&format=1&location=45.013640,-97.123560!&location-radius=112.500000m");
+	g_free (query_uri);
+
+	gdata_youtube_query_set_location (query, G_MAXDOUBLE, 0.6672, 52.8, TRUE);
+
+	query_uri = gdata_query_get_query_uri (GDATA_QUERY (query), "http://example.com");
+	g_assert_cmpstr (query_uri, ==, "http://example.com?q=q&time=all_time&safeSearch=none&format=1&location=!");
+	g_free (query_uri);
+
+	gdata_youtube_query_set_location (query, G_MAXDOUBLE, G_MAXDOUBLE, 0.0, FALSE);
+
+	query_uri = gdata_query_get_query_uri (GDATA_QUERY (query), "http://example.com");
+	g_assert_cmpstr (query_uri, ==, "http://example.com?q=q&time=all_time&safeSearch=none&format=1");
+	g_free (query_uri);
+
+	/* Language */
+	gdata_youtube_query_set_language (query, "fr");
+	g_assert_cmpstr (gdata_youtube_query_get_language (query), ==, "fr");
+
+	gdata_youtube_query_set_order_by (query, "relevance_lang_fr");
+	g_assert_cmpstr (gdata_youtube_query_get_order_by (query), ==, "relevance_lang_fr");
+
+	gdata_youtube_query_set_restriction (query, "192.168.0.1");
+	g_assert_cmpstr (gdata_youtube_query_get_restriction (query), ==, "192.168.0.1");
+
+	query_uri = gdata_query_get_query_uri (GDATA_QUERY (query), "http://example.com");
+	g_assert_cmpstr (query_uri, ==, "http://example.com?q=q&time=all_time&safeSearch=none&format=1&lr=fr&orderby=relevance_lang_fr&restriction=192.168.0.1");
+	g_free (query_uri);
+
+	gdata_youtube_query_set_safe_search (query, GDATA_YOUTUBE_SAFE_SEARCH_STRICT);
+	g_assert_cmpuint (gdata_youtube_query_get_safe_search (query), ==, GDATA_YOUTUBE_SAFE_SEARCH_STRICT);
+
+	query_uri = gdata_query_get_query_uri (GDATA_QUERY (query), "http://example.com");
+	g_assert_cmpstr (query_uri, ==, "http://example.com?q=q&time=all_time&safeSearch=strict&format=1&lr=fr&orderby=relevance_lang_fr&restriction=192.168.0.1");
+	g_free (query_uri);
+
+	gdata_youtube_query_set_sort_order (query, GDATA_YOUTUBE_SORT_ASCENDING);
+	g_assert_cmpuint (gdata_youtube_query_get_sort_order (query), ==, GDATA_YOUTUBE_SORT_ASCENDING);
+
+	gdata_youtube_query_set_age (query, GDATA_YOUTUBE_AGE_THIS_WEEK);
+	g_assert_cmpuint (gdata_youtube_query_get_age (query), ==, GDATA_YOUTUBE_AGE_THIS_WEEK);
+
+	gdata_youtube_query_set_uploader (query, GDATA_YOUTUBE_UPLOADER_PARTNER);
+	g_assert_cmpuint (gdata_youtube_query_get_uploader (query), ==, GDATA_YOUTUBE_UPLOADER_PARTNER);
+
+	/* Check the built URI with a normal feed URI */
+	query_uri = gdata_query_get_query_uri (GDATA_QUERY (query), "http://example.com");
+	g_assert_cmpstr (query_uri, ==, "http://example.com?q=q&time=this_week&safeSearch=strict&format=1&lr=fr&orderby=relevance_lang_fr&restriction=192.168.0.1&sortorder=ascending&uploader=partner");
+	g_free (query_uri);
+
+	/* …and with a feed URI with pre-existing arguments */
+	query_uri = gdata_query_get_query_uri (GDATA_QUERY (query), "http://example.com?foobar=shizzle");
+	g_assert_cmpstr (query_uri, ==, "http://example.com?foobar=shizzle&q=q&time=this_week&safeSearch=strict&format=1&lr=fr&orderby=relevance_lang_fr&restriction=192.168.0.1&sortorder=ascending&uploader=partner");
+	g_free (query_uri);
+
+	g_object_unref (query);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -393,6 +599,9 @@ main (int argc, char *argv[])
 	if (g_test_slow () == TRUE)
 		g_test_add_func ("/youtube/upload/simple", test_upload_simple);
 	g_test_add_func ("/youtube/parsing/app:control", test_parsing_app_control);
+	g_test_add_func ("/youtube/parsing/comments/feedLink", test_parsing_comments_feed_link);
+	g_test_add_func ("/youtube/parsing/yt:recorded", test_parsing_yt_recorded);
+	g_test_add_func ("/youtube/query/uri", test_query_uri);
 
 	retval = g_test_run ();
 	if (service != NULL)
